package br.com.simpleunit;

import br.com.simpleunit.objects.FinalResults;

/**
 * This class provides a restrict view of Core class instance of framework.
 * Basically, it is used by the classes responsible for the generation of output reports
 * 
 * @author Robson Paulo Kraemer (rpkraemer@gmail.com)
 *
 */
public class CoreWrapper {

	/**
	 * Reference variable for SimpleUnit core class
	 */
	private final SimpleUnit core;
	
	/**
	 * Receives SimpleUnit core instance and encapsulates some of your information
	 * @param core
	 */
	public CoreWrapper(SimpleUnit core) {
		this.core = core;
	}
	
	/**
	 * Used to retrieve the final results generated by the core
	 * @return FinalResults instance
	 */
	public FinalResults getFinalResults() {
		return core.getFinalResults();
	}
	
	/**
	 * Used to retrieve the total execution elapsed time
	 * @return total execution elapsed time
	 */
	public long timeElapsed() {
		return core.timeElapsed();
	}
	
	/**
	 * Used to sinalize referenced Core instance that report compilation finished
	 */
	public void sinalizeEndExecution() {
		this.core.sinalizeEndExecution();
	}
}
